#!/usr/bin/ruby
# coding: UTF-8

#-------------------------------------------------------------------------------
# Author: Dr. Heinz Breinlinger
#
# This program is free software; you can redistribute it and/or modify it under the 
# terms of the GNU General Public License as published by the Free Software Foundation; 
# either version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY 
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with 
# this program; if not, see <http://www.gnu.org/licenses/>.
#-------------------------------------------------------------------------------
 
require 'rubygems'
require 'getoptlong'

#{{{{die
def die(message)
  puts("#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
  puts("mycroncheck.rb: Error: #{message}")
  puts("#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
  exit 1
end
#}}}}
#{{{{Variablen und Konstanten
LINENUM="lineNum"
STARTTIME="start"
ENDTIME="end"
DEVICE="device"
TITLE="title"
MAXRUNNING=2
MAXANTENNE=2
MAXSAT=2
infile=""
currYear=Time.now.year
currMonth=Time.now.month
#----------------------------------------------------------------
# Array aller Hashes aus Zeilennummer und zugehoerigen Daten, sowie
# sortierte Variante
#----------------------------------------------------------------
episodeHashArr=Array.new()
sortedEpisodeHashArr=Array.new()
#----------------------------------------------------------------
# Hash aus Zeilennummern und Episoden-Hashes
#----------------------------------------------------------------
episodeHashHash=Hash.new()
#----------------------------------------------------------------
# Array aller zum Betrachtungszeitpunkt laufenden Sendungen
#----------------------------------------------------------------
runningArr=Array.new()
#----------------------------------------------------------------
# Array aller Start-/Endzeiten ohne Doubletten
#----------------------------------------------------------------
timesArr=Array.new()
#}}}}
#{{{{Command-Line-Options
options=GetoptLong.new(
  ['--help', '-h', GetoptLong::NO_ARGUMENT],
  ['--infile', '-i', GetoptLong::REQUIRED_ARGUMENT]
)
options.each{ |opt,arg|
  case opt
    when '--help'
      p "Usage: mycroncheck.rb --infile|i <infile>"
    when '--infile'
      infile=arg
  end
}
#}}}}
#{{{{Hauptverarbeitung
unless(File.exists?(infile))
  die("Kann Datei \"#{infile}\" nicht finden")
end
#----------------------------------------------
infile=File.open(infile,"r")
lineNum=0
#{{{{crontab einlesen
while(line=infile.gets)
  lineNum+=1
  #----------------------------------------------------------
  # Crontab-Zeile mit mytv-Eintragung ?
  # Beispielzeile
  #   30 19 19 12 * /usr/bin/mytv -g antenne -r "arte" -d 00:55:00 -n "Europas_hoher_Norden_Daenemark"
  #----------------------------------------------------------
  if(line.match(/^(\d{2})\s+(\d{2})\s+(\d{2})\s+(\d{2}).+mytv.+-g\s+(antenne|sat).+(\d{2}):(\d{2}):00.+-n\s+"(.+)"/))
    month=$4; day=$3; hour=$2; minute=$1
    #-------------------------------------------------------
    # zugehoeriges Jahr berechnen
    # Cave: Reine Heuristik, welche nur "in fast allen" Faellen
    # sinnvolle Ergebnisse liefert.
    #-------------------------------------------------------
    if(month.to_i >= currMonth)
      year=currYear
    else
      year=currYear+1
    end
    startTime=Time.gm(year,month,day,hour,minute)
    device=$5
    duration=(($6.to_i*60)+$7.to_i)*60
    title=$8
    endTime=Time.at(startTime+duration).gmtime
    #--------------------------------------------------------
    # Array aus crontab-Eintraegen (Hashes) bauen
    #--------------------------------------------------------
    episodeHash={LINENUM => lineNum.to_s,
                 STARTTIME => startTime,
                 ENDTIME => endTime}
    episodeHashArr.push(episodeHash)
    #--------------------------------------------------------
    # Hash aus Zeilennummern und Episoden-Hashes bilden
    #--------------------------------------------------------
    episodeHashHash[lineNum.to_s]={DEVICE => device,
                                   TITLE => title}
    #--------------------------------------------------------
    # Zeiten, sofern noch nicht vorhanden, in Vektor aller
    # Start-/Endzeiten eintragen.
    #--------------------------------------------------------
    unless(timesArr.include?(startTime))
      timesArr.push(startTime)
    end
    unless(timesArr.include?(endTime))
      timesArr.push(endTime)
    end
  end
end
#----------------------------------------------
infile.close()
#}}}}
#----------------------------------------------
# Array aller Start-/Endzeiten sortieren
#----------------------------------------------
timesArr.sort!
#----------------------------------------------
# Array aller Hashes der Episoden nach Anfangszeit sortieren
#----------------------------------------------
sortedEpisodeHashArr=episodeHashArr.sort_by{ |episodeHash|
  episodeHash[STARTTIME]
}
#{{{{Parallel laufende Sendungen ermitteln und Randbedingungen pruefen
#-----------------------------------------------------------------------------
# Fuer jeden Start-/Endzeitpunkt der Gesamtliste ...
#-----------------------------------------------------------------------------
timesArr.each{ |time|
  #---------------------------------------------------------------------------
  # werden fuer jeden Sendungseintrag geprueft, ...
  #---------------------------------------------------------------------------
  sortedEpisodeHashArr.each{ |episodeHash|
    startTime=episodeHash[STARTTIME]
    endTime=episodeHash[ENDTIME]
    lineNum=episodeHash[LINENUM]
    #-------------------------------------------------------------------------
    # ob die Sendung zu diesem Zeitpunkt laeuft ...
    #-------------------------------------------------------------------------
    if((startTime <= time) && (endTime >= time))
      #-----------------------------------------------------------------------
      # selbige, sofern noch nicht vorhanden, genau dann in den Vektor der laufenden
      # Sendungen aufgenommen, ...
      #-----------------------------------------------------------------------
      unless(runningArr.include?(lineNum))
        runningArr.push(lineNum)
      end
    #------------------------------------------------------------------------
    # oder, sofern vorhanden, aber nicht (mehr) laufend, aus selbigem entfernt.
    #------------------------------------------------------------------------
    elsif(runningArr.include?(lineNum))
      runningArr.delete(lineNum)
    end
  }
  #{{{{Randbedingungen pruefen
  #-----------------------------------------------------------------------
  # Falls sich mehr Sendungen ueberlappen als erlaubt => Warnmeldung
  #-----------------------------------------------------------------------
  if(runningArr.length() > MAXRUNNING)
    puts("---------------------------------------------------")
    runningArr.each{ |lineNum|
      title=episodeHashHash[lineNum][TITLE]
      puts("#{time}: Zu viele Ueberlappungen: #{title} - Zeile #{lineNum}")
    }
    puts("---------------------------------------------------")
  end
  #-----------------------------------------------------------------------
  # Falls mehr Zugriffe auf bestimmtes Aufnahmegeraet als erlaubt => Warnung
  #-----------------------------------------------------------------------
  numSat=0
  numAntenne=0
  runningArr.each{ |lineNum|
    device=episodeHashHash[lineNum][DEVICE]
    if(device == "sat")
      numSat+=1
    elsif(device == "antenne")
      numAntenne+=1
    end
  }
  #----------------------------------------------------------------------
  if(numSat > MAXSAT)
    runningArr.each{ |lineNum|
      title=episodeHashHash[lineNum][TITLE]
      puts("---------------------------------------------------")
      puts("#{time}: DVB-S ueberschritten: #{title} - Zeile #{lineNum}")
      puts("---------------------------------------------------")
    }
  end
  #----------------------------------------------------------------------
  if(numAntenne > MAXANTENNE)
    runningArr.each{ |lineNum|
      title=episodeHashHash[lineNum][TITLE]
      puts("---------------------------------------------------")
      puts("#{time}: DVB-T ueberschritten: #{title} - Zeile #{lineNum}")
      puts("---------------------------------------------------")
    }
  end
  #----------------------------------------------------------------------
  #}}}}
}
#}}}}
#}}}}
