#!/usr/bin/ruby
# coding: UTF-8
 
#-------------------------------------------------------------------------------
# Author: Dr. Heinz Breinlinger
#
# This program is free software; you can redistribute it and/or modify it under the 
# terms of the GNU General Public License as published by the Free Software Foundation; 
# either version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY 
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with 
# this program; if not, see <http://www.gnu.org/licenses/>.
#-------------------------------------------------------------------------------

require 'rubygems'
require 'rexml/document'
require 'date'
require 'getoptlong'

#########################################################
def die(message)
  puts("#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
  puts("myepgcron.rb: Error: #{message}")
  puts("#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
  exit 1
end
#########################################################
def pad(value)
  if (value < 10)
    return String.new("0#{value}")
  else
    return value
  end
end
#########################################################
infile=""
options=GetoptLong.new(
  ['--help', '-h', GetoptLong::NO_ARGUMENT],
  ['--infile', '-i', GetoptLong::REQUIRED_ARGUMENT]
)
options.each{ |opt,arg|
  case opt
    when '--help'
      p "Usage: myepgcron.rb --infile|i <infile>"
    when '--infile'
      infile=arg
  end
}
#########################################################
unless(File.exists?(infile))
  die ("Kann Datei \"#{infile}\" nicht finden")
end
#----------------------------------------------
file=File.new(infile)
puts("Parsing #{infile}")
begin
  doc=REXML::Document.new file
rescue => exception
  die("#{$!}/#{exception.message}")
end
puts("Done ...")
#----------------------------------------------
channel=""
number=""
start=""
duration=""
title=""
trailer=600 # Aufzeichungsnachlauf von 10 Minuten (600 Sek.)
numTagsToFind=4   # Anzahl zu findender Knoten unterhalb "pack/data" für break-condition
dvbTChannel=["ard","zdf","sat1","prosieben","kabeleins","arte","3sat","phoenix","bayern",
             "bralpha","einsplus","einsextra","zdfinfo","n24","swr","hse24"]
#----------------------------------------------
doc.elements.each("pack/data"){ |data|
  numTagsFound=0
  data.elements.each("*"){ |element|
    if (numTagsFound >= numTagsToFind)
      break
    end
    elementText=element.to_s
    if(elementText.match(/<d2>/))
      #--------------------------------------------------------
      # Kanal aus Text mit fortlaufender Nummer der Form
      #   <d2>zdf #237</d2>
      # extrahieren
      #--------------------------------------------------------
      channel=element.text.sub(/\s*#\d+/,"")
      number=element.text.match(/#\d+/)
      numTagsFound+=1
    elsif(elementText.match(/<d4>/))
      start=DateTime.parse(element.text)
      numTagsFound+=1
    elsif(elementText.match(/<d7>/))
      duration=Time.at(element.text.to_i*60+trailer).gmtime.strftime('%R:%S')
      numTagsFound+=1
    elsif(elementText.match(/<d19>/))
      title=element.text
      title.gsub!(/&Atilde;$curren;/,"ae")
      title.gsub!(/&Atilde;$para;/,"oe")
      title.gsub!(/&Atilde;$frac14;/,"ue")
      title.gsub!(/ä/,"ae")
      title.gsub!(/ö/,"oe")
      title.gsub!(/ü/,"ue")
      title.gsub!(/Ä/,"Ae")
      title.gsub!(/Ö/,"Oe")
      title.gsub!(/Ü/,"Ue")
      title.gsub!(/ß/,"ss")
      title.gsub!(/&auml;/,"ae")
      title.gsub!(/&ouml;/,"oe")
      title.gsub!(/&uuml;/,"ue")
      title.gsub!(/&Auml;/,"Ae")
      title.gsub!(/&Ouml;/,"Oe")
      title.gsub!(/&Uuml;/,"Ue")
      title.gsub!(/&szlig;/,"ss")
      title.gsub!(/\s+/,"_")
      title.gsub!(/[^[:alnum:]_-]/,"")
      numTagsFound+=1
    end
  }
  #-----------------------------------------------------------------------------
  # Netstream-DTT verwenden, wo immer moeglich
  #-----------------------------------------------------------------------------
  if(dvbTChannel.include?(channel))
    device="antenne"
  else
    device="sat"
  end
  puts("#{number}: #{pad(start.min)} #{pad(start.hour)} #{pad(start.day)} #{pad(start.month)} * /usr/bin/mytv -g #{device} -r \"#{channel}\" -d #{duration} -n \"#{title}\"")
}
